<?php
declare(strict_types=1);

namespace Intergo\LaravelAuthMiddleware\Http\Middleware;

use Illuminate\Support\Facades\Session;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\LaravelAuthMiddleware\Models\AuthUser;
use Intergo\BaseClient\Responses\BaseResponse;
use Intergo\Log\GrayLogHandler;
use Intergo\LaravelAuthMiddleware\Services\Microservices\AuthService;
use Intergo\LaravelAuthMiddleware\Traits\ResponseTrait;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Closure;

abstract class AbstractAuthMiddleware
{
    use ResponseTrait;

    protected Request $request;

    /**
     * @param Request $request
     * @param Closure $next
     * @param string $params
     * @return BaseResponseInterface
     */
    public function handle(Request $request, Closure $next, string $params = '')
    {
        $this->setRequest($request);
        $result = $this->authenticateUserThroughAuth();
        if ($result->isSuccessful()) {
            $this->addUserToRequest($result->getData());
        }
        return $result;
    }
    /**
     * @param Request $request
     */
    protected function setRequest(Request $request)
    {
        $this->request = $request;
    }

    /**
     * @return BaseResponseInterface
     */
    protected function authenticateUserThroughAuth()
    {
        $token = Session::get(AuthService::SESSION_JWT_TOKEN) ?? $this->request->bearerToken() ?? $this->request->api_key;
        if (empty($token)) {
            $message = 'API Authenticate: Token is missing';
            $loggingHandler = app(GrayLogHandler::class);
            $loggingTag = env('AUTH_TAG',env('APP_NAME','AuthService'));
            $loggingHandler->setTag($loggingTag);
            $loggingHandler->error($message);
            return new BaseResponse(['success' => false, 'message' => $message, 'statusCode' => 401]);
        }
        //the default IP is in cases we can't determine the api from the request
        $ip = !empty($this->request->ip()) ? $this->request->ip() : AuthService::DEFAULT_IP;
        $authService = new AuthService();
        return $authService->authenticate($token, $ip, '');
    }

    /**
     * @param $user
     * @return void
     */
    protected function addUserToRequest($user)
    {
        $user['_id'] = $user['id'];
        $userCollection = AuthUser::hydrate(["user" => $user]);
        $user = $userCollection->first();
        \Auth::setUser($user); // This is setting the user to laravel Auth resolver, to be able to use it everywhere
        $this->request->user = $user;
    }
}
